import logging

from spaceone.core import config
from spaceone.core.error import *
from spaceone.repository.model import Plugin
from spaceone.repository.manager.plugin_manager import PluginManager

__all__ = ['LocalPluginManager']

_LOGGER = logging.getLogger(__name__)
_REGISTRY_CONNECTOR_MAP = {
    'DOCKER_HUB': 'DockerHubConnector',
    'AWS_PUBLIC_ECR': 'AWSPublicECRConnector',
    'HARBOR': 'HarborConnector'
}


class LocalPluginManager(PluginManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.plugin_model: Plugin = self.locator.get_model("Plugin")

    def register_plugin(self, params):
        def _rollback(plugin_vo):
            plugin_vo.delete()

        plugin_vo = self.plugin_model.create(params)
        self.transaction.add_rollback(_rollback, plugin_vo)

        return plugin_vo

    def update_plugin(self, params):
        plugin_vo = self.get_plugin(params['plugin_id'], params['domain_id'])
        return self.update_plugin_by_vo(params, plugin_vo)

    def enable_plugin(self, plugin_id, domain_id):
        plugin_vo = self.get_plugin(plugin_id, domain_id)
        return self.update_plugin_by_vo({'state': 'ENABLED'}, plugin_vo)

    def disable_plugin(self, plugin_id, domain_id):
        plugin_vo = self.get_plugin(plugin_id, domain_id)
        return self.update_plugin_by_vo({'state': 'DISABLED'}, plugin_vo)

    def update_plugin_by_vo(self, params, plugin_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Plugin Data : {old_data["name"]} ({old_data["plugin_id"]})')
            plugin_vo.update(old_data)

        self.transaction.add_rollback(_rollback, plugin_vo.to_dict())
        return plugin_vo.update(params)

    def delete_plugin(self, plugin_id, domain_id):
        plugin_vo = self.plugin_model.get(domain_id=domain_id, plugin_id=plugin_id)
        plugin_vo.delete()

    def get_plugin(self, plugin_id, domain_id, only=None):
        if domain_id:
            plugin_vo = self.plugin_model.get(plugin_id=plugin_id, domain_id=domain_id, only=only)
        else:
            plugin_vo = self.plugin_model.get(plugin_id=plugin_id, only=only)

        return plugin_vo

    def list_plugins(self, query):
        return self.plugin_model.query(**query)

    def stat_plugins(self, query):
        return self.plugin_model.stat(**query)

    def get_plugin_versions(self, plugin_id, domain_id):
        """ Get version of image

        version: tag list of docker image
        create RegistryConnector
        call get_tags()

        Returns:
            A list of docker tag
        """
        plugin_vo: Plugin = self.get_plugin(plugin_id, domain_id)

        registry_url = config.get_global('REGISTRY_URL_MAP', {}).get(plugin_vo.registry_type)

        try:
            connector = self.locator.get_connector(_REGISTRY_CONNECTOR_MAP[plugin_vo.registry_type])
            tags = connector.get_tags(registry_url, plugin_vo.image, plugin_vo.registry_config)
        except Exception as e:
            _LOGGER.error(f'[get_plugin_versions] get_tags error: {e}', exc_info=True)
            raise e
        else:
            return tags



import logging
from abc import abstractmethod

from spaceone.core.manager import BaseManager

__all__ = ['PluginManager']
_LOGGER = logging.getLogger(__name__)


class PluginManager(BaseManager):

    def register_plugin(self, params):
        pass

    def update_plugin(self, params):
        pass

    def enable_plugin(self, plugin_id, domain_id):
        pass

    def disable_plugin(self, plugin_id, domain_id):
        pass

    def delete_plugin(self, plugin_id, domain_id):
        pass

    @abstractmethod
    def get_plugin(self, plugin_id, domain_id, only=None):
        pass

    @abstractmethod
    def list_plugins(self, query):
        pass

    @abstractmethod
    def stat_plugins(self, query):
        pass

    @abstractmethod
    def get_plugin_versions(self, plugin_id, domain_id):
        pass



from spaceone.core.error import *


class ERROR_NO_REPOSITORY(ERROR_BASE):
    _message = 'Repository does not exists.'


class ERROR_REPOSITORY_BACKEND(ERROR_BASE):
    _status_code = 'INTERNAL'
    _message = 'Repository backend has problem. ({host})'


class ERROR_AWS_ECR_TOKEN(ERROR_BASE):
    _message = 'Failed to get ECR TOKEN, check IAM policy'



import functools
from spaceone.api.repository.v1 import repository_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.repository.model.repository_model import Repository

__all__ = ['RepositoryInfo','RepositoriesInfo']


def RepositoryInfo(repository_vo: Repository, minimal=False):
    info = {
        'repository_id': repository_vo.repository_id,
        'name': repository_vo.name,
        'repository_type': repository_vo.repository_type
    }
    if not minimal:
        info.update({
            'endpoint': repository_vo.endpoint,
            'version': repository_vo.version,
            'secret_id': repository_vo.secret_id,
            'created_at': utils.datetime_to_iso8601(repository_vo.created_at)
        })

    return repository_pb2.RepositoryInfo(**info)


def RepositoriesInfo(repo_vos, total_count, **kwargs):
    return repository_pb2.RepositoriesInfo(results=list(
        map(functools.partial(RepositoryInfo, **kwargs), repo_vos)), total_count=total_count)



from spaceone.repository.manager.repository_manager import RepositoryManager


class LocalRepositoryManager(RepositoryManager):

    def register_repository(self, params):
        # Assume there is only one local repository

        return self.repo_model.create(params)



DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    'default': {
        'db': 'repository',
        'host': 'localhost',
        'port': 27017,
        'username': 'repository',
        'password': ''
    }
}

CACHES = {
    'default': {},
    'local': {
        'backend': 'spaceone.core.cache.local_cache.LocalCache',
        'max_size': 128,
        'ttl': 300
    }
}

CONNECTORS = {
    'AWSPublicECRConnector': {
        'aws_access_key_id': '',
        'aws_secret_access_key': '',
        'region_name': ''
    },
    'DockerHubConnector': {},
    'HarborConnector': {
        'base_url': '',
        'token': '',
        'verify': True
    },
    'RemoteRepositoryConnector': {},
    'SpaceConnector': {
        'backend': 'spaceone.core.connector.space_connector.SpaceConnector',
        'endpoints': {
            'identity': 'grpc://identity:50051',
            'secret': 'grpc://secret:50051'
        }
    }
}

REGISTRY_URL_MAP = {
    'DOCKER_HUB': 'registry.hub.docker.com',
    'AWS_PUBLIC_ECR': 'public.ecr.aws',
    'HARBOR': ''
}

ROOT_TOKEN = ""
ROOT_TOKEN_INFO = {}




import logging
from spaceone.repository.manager.policy_manager import PolicyManager
from spaceone.repository.connector.remote_repository_connector import RemoteRepositoryConnector

__all__ = ['RemotePolicyManager']

_LOGGER = logging.getLogger(__name__)


class RemotePolicyManager(PolicyManager):
    """
    self.repository (=repository_vo)
    Remote Policy make gRPC call to remote repository (like marketplace)
    If connector gets policy_info, this is gRPC message.
    """

    def get_policy(self, policy_id, domain_id, only=None):
        conn = self._get_conn_from_repository()
        connector: RemoteRepositoryConnector = self.locator.get_connector('RemoteRepositoryConnector', conn=conn)

        policy_info = connector.get_policy(policy_id, only)
        return self._get_updated_policy_info(policy_info)

    def list_policies(self, query):
        conn = self._get_conn_from_repository()
        connector: RemoteRepositoryConnector = self.locator.get_connector('RemoteRepositoryConnector', conn=conn)

        response = connector.list_policies(query, self.repository.repository_id)
        _LOGGER.debug(f'[remote list_policy] count: {response.total_count}')

        for policy_info in response.results:
            # Warning:
            # This is side effect coding, since policy_vo is protobuf message
            self._get_updated_policy_info(policy_info)

        return response.results, response.total_count

    def stat_policies(self, query):
        raise NotImplementedError('Remote repository is not supported.')

    def _get_conn_from_repository(self):
        conn = {'endpoint': self.repository.endpoint}
        return conn

    def _get_updated_policy_info(self, policy_info):
        """
        policy_info is Protobuf Message
        We want to change our policy_info (especially repository_info)

        Args:
            - policy_info: protobuf message
        """
        # domain_id is remote repository's domain_id
        # change to local repository's domain_id  
        # There is no way to find domain_id
        # TODO: policy_info.domain_id = self.repository.domain_id

        policy_info.repository_info.name = self.repository.name
        policy_info.repository_info.repository_type = self.repository.repository_type
        return policy_info



from mongoengine import *
from datetime import datetime

from spaceone.core.error import *
from spaceone.core.model.mongo_model import MongoModel

__all__ = ['Repository']


class Repository(MongoModel):
    repository_id = StringField(max_length=40, generate_id='repo')
    name = StringField(max_length=255)
    repository_type = StringField(max_length=255, choices=['local', 'remote'])
    state = StringField(max_length=20, default='ENABLED')
    endpoint = StringField(max_length=255)
    version = StringField(max_length=16)
    secret_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)
    deleted_at = DateTimeField(default=None, null=True)

    meta = {
        'updatable_fields': [
            'name',
            'secret_id',
            'state',
            'deleted_at'
        ],
        'minimal_fields': [
            'repository_id',
            'name',
            'repository_type'
        ],
        'ordering': ['name'],
        'indexes': [
            'repository_id',
            'name',
            'repository_type'
        ]
    }

    @queryset_manager
    def objects(doc_cls, queryset):
        return queryset.filter(state__ne='DELETED')

    @classmethod
    def create(cls, data):
        rr_vos = cls.filter(name=data['name'])
        if rr_vos.count() > 0:
            raise ERROR_NOT_UNIQUE(key='name', value=data['name'])

        return super().create(data)

    def update(self, data):
        if 'name' in data:
            rr_vos = self.filter(name=data['name'], repository_id__ne=self.repository_id)
            if rr_vos.count() > 0:
                raise ERROR_NOT_UNIQUE(key='name', value=data['name'])

        return super().update(data)

    def delete(self):
        self.update({
            'state': 'DELETED',
            'deleted_at': datetime.utcnow()
        })



import logging
from abc import abstractmethod

from spaceone.core.manager import BaseManager
from spaceone.repository.model.repository_model import Repository

_LOGGER = logging.getLogger(__name__)


class RepositoryManager(BaseManager):
    def __init__(self, transaction, **kwargs):
        super().__init__(transaction, **kwargs)
        self.repo_model: Repository = self.locator.get_model('Repository')

    @abstractmethod
    def register_repository(self, params):
        pass

    def delete_repository(self, repository_id):
        repo_vo = self.repo_model.get(repository_id=repository_id)
        repo_vo.delete()

    def update_repository(self, params):
        def _rollback(old_data):
            repo_vo.update(old_data)

        repo_vo = self.repo_model.get(repository_id=params['repository_id'])
        self.transaction.add_rollback(_rollback, repo_vo.to_dict())
        
        repo_vo.update(params)
        return repo_vo

    def get_repository(self, repository_id, only=None):
        return self.repo_model.get(repository_id=repository_id, only=only)

    def filter_repositories(self, **conditions):
        return self.repo_model.filter(**conditions)

    def list_repositories(self, query):
        return self.repo_model.query(**query)

    def stat_repositories(self, query):
        return self.repo_model.stat(**query)

    def get_all_repositories(self, repository_id=None):
        query = {
            'sort': {
                'key': 'repository_type'
            },
            'filter': []
        }

        if repository_id:
            query['filter'].append({'k': 'repository_id', 'v': repository_id, 'o': 'eq'})

        repo_vos, total_count = self.list_repositories(query)
        _LOGGER.debug(f'[list_repositories_by_id] Number of repositories: {total_count}')

        return repo_vos

    def get_local_repository(self):
        repo_vos = self.filter_repositories(repository_type='local')
        local_repo_count = repo_vos.count()
        _LOGGER.debug(f'[get_local_repository] local_repo_count: {local_repo_count}')
        if local_repo_count > 0:
            return repo_vos[0]
        else:
            return None





import abc
import requests
import logging
import boto3
from distutils.version import StrictVersion

from spaceone.core.connector import BaseConnector
from spaceone.repository.error import *

__all__ = ['DockerHubConnector', 'AWSPublicECRConnector', 'HarborConnector']

_LOGGER = logging.getLogger(__name__)
_DEFAULT_PAGE_SIZE = 10


class RegistryConnector(BaseConnector):

    @abc.abstractmethod
    def get_tags(self, registry_url, image, config):
        pass


class DockerHubConnector(RegistryConnector):

    def get_tags(self, registry_url, image, config):
        # url = f'https://{registry_url}/v2/repositories/{image}/tags?page_size={_DEFAULT_PAGE_SIZE}'
        url = f'https://{registry_url}/v2/repositories/{image}/tags'

        response = requests.get(url)

        if response.status_code == 200:
            image_tags = []
            results = response.json().get('results', [])
            for tag in results:
                image_tags.append(tag['name'])

            return image_tags

        else:
            raise ERROR_NO_IMAGE_IN_REGISTRY(registry_type='DOCKER_HUB', image=image)


class HarborConnector(RegistryConnector):

    def get_tags(self, registry_url, image, config):
        base_url = self.config.get('base_url')
        verify = self.config.get('verify', True)

        headers = {
            'authorization': f'Basic {self.config["token"]}'
        }

        url = f'{base_url}/v2/{image}/tags/list'

        response = requests.get(url, headers=headers, verify=verify)

        if response.status_code == 200:
            image_tags = response.json().get('tags', [])
            image_tags.sort(key=StrictVersion, reverse=True)
            return image_tags

        raise ERROR_NO_IMAGE_IN_REGISTRY(registry_type='HARBOR', image=image)


class AWSPublicECRConnector(RegistryConnector):

    _DEFAULT_REGION_NAME = 'us-east-1'

    def __init__(self, transaction, config):
        super().__init__(transaction, config)

        aws_access_key_id = self.config.get('aws_access_key_id')
        aws_secret_access_key = self.config.get('aws_secret_access_key')

        if not all([aws_access_key_id, aws_secret_access_key]):
            raise ERROR_CONNECTOR_CONFIGURATION(connector='AWSECRConnector')

        self.client = boto3.client('ecr-public',
                                   aws_access_key_id=aws_access_key_id,
                                   aws_secret_access_key=aws_secret_access_key,
                                   region_name=self._DEFAULT_REGION_NAME)

    def get_tags(self, registry_url, image, config):
        # TODO: change docker registry v2 api

        account_id = config.get('account_id')
        ecr_image = self._parse_ecr_image(image)

        try:
            response = self.client.describe_images(repositoryName=ecr_image, registryId=account_id)

            image_tags = []
            results = response.get('imageTagDetails', [])
            sorted_results = sorted(results, key=lambda k: k['createdAt'], reverse=True)
            for tag in sorted_results:
                image_tags.append(tag['imageTag'])

            return image_tags

        except Exception as e:
            _LOGGER.error(f'[get_tags] boto3 describe_image_tags error: {e}')
            raise ERROR_NO_IMAGE_IN_REGISTRY(registry_type='AWS_PUBLIC_ECR', image=image)

    @staticmethod
    def _parse_ecr_image(image):
        return '/'.join(image.split('/')[1:])


if __name__ == '__main__':
    docker_hub_conn = DockerHubConnector()
    tags = docker_hub_conn.get_tags('registry.hub.docker.com', 'spaceone/repository')
    print(tags)



import functools
from spaceone.api.repository.v1 import plugin_pb2
from spaceone.core import config
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.repository.model.plugin_model import Plugin
from spaceone.repository.info.repository_info import RepositoryInfo

__all__ = ['PluginInfo', 'PluginsInfo', 'VersionsInfo']


def PluginInfo(plugin_vo, minimal=False):
    info = {
        'plugin_id': plugin_vo.plugin_id,
        'name': plugin_vo.name,
        'state': plugin_vo.state,
        'image': plugin_vo.image,
        'service_type': plugin_vo.service_type,
        'registry_type': plugin_vo.registry_type,
        'provider': plugin_vo.provider
    }
    if not minimal:
        info.update({
            'registry_config': change_struct_type(plugin_vo.registry_config),
            'capability': change_struct_type(plugin_vo.capability),
            'template': change_struct_type(plugin_vo.template),
            'labels': change_list_value_type(plugin_vo.labels),
            'project_id': plugin_vo.project_id,
            'domain_id': plugin_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(plugin_vo.created_at) or plugin_vo.created_at,
            'updated_at': utils.datetime_to_iso8601(plugin_vo.updated_at) or plugin_vo.updated_at
            })

        if isinstance(plugin_vo, plugin_pb2.PluginInfo):
            info['tags'] = plugin_vo.tags
            info['registry_url'] = plugin_vo.registry_url
        else:
            info['tags'] = change_struct_type(plugin_vo.tags)
            if plugin_vo.registry_type:
                info['registry_url'] = config.get_global('REGISTRY_URL_MAP', {}).get(plugin_vo.registry_type)

        # WARNING
        # Based on local_plugin or remote_plugin
        # vo has different repository or repository_info field
        if getattr(plugin_vo, 'repository', None):
            info.update({
                'repository_info': RepositoryInfo(plugin_vo.repository, minimal=True)})
        if getattr(plugin_vo, 'repository_info', None):
            info.update({
                'repository_info': RepositoryInfo(plugin_vo.repository_info, minimal=True)})

    return plugin_pb2.PluginInfo(**info)


def PluginsInfo(plugin_vos, total_count, **kwargs):
    return plugin_pb2.PluginsInfo(results=list(
        map(functools.partial(PluginInfo, **kwargs), plugin_vos)), total_count=total_count)


def VersionsInfo(version_list):
    return plugin_pb2.VersionsInfo(results=version_list, total_count=len(version_list))



PROTO = {
    'spaceone.repository.api.v1.repository': ['Repository'],
    'spaceone.repository.api.v1.plugin': ['Plugin'],
    'spaceone.repository.api.v1.schema': ['Schema'],
    'spaceone.repository.api.v1.policy': ['Policy'],
}



import functools
from google.protobuf.struct_pb2 import Struct
from spaceone.api.repository.v1 import schema_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.repository.model.schema_model import Schema
from spaceone.repository.info.repository_info import RepositoryInfo

__all__ = ['SchemaInfo', 'SchemasInfo']


def SchemaInfo(schema_vo, minimal=False):
    info = {
        'name': schema_vo.name,
        'service_type': schema_vo.service_type
    }

    if not minimal:
        info.update({
            'schema': change_struct_type(schema_vo.schema),
            'labels': change_list_value_type(schema_vo.labels),
            'tags': change_struct_type(schema_vo.tags),
            'project_id': schema_vo.project_id,
            'domain_id': schema_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(schema_vo.created_at) or schema_vo.created_at,
            'updated_at': utils.datetime_to_iso8601(schema_vo.updated_at) or schema_vo.updated_at
            })
        # WARNING
        # Based on local_schema or remote_schema
        # vo has different repository or repository_info field
        if getattr(schema_vo, 'repository', None):
            info.update({
                'repository_info': RepositoryInfo(schema_vo.repository, minimal=True)})
        if getattr(schema_vo, 'repository_info', None):
            info.update({
                'repository_info': RepositoryInfo(schema_vo.repository_info, minimal=True)})

    return schema_pb2.SchemaInfo(**info)


def SchemasInfo(schema_vos, total_count, **kwargs):
    return schema_pb2.SchemasInfo(results=list(
        map(functools.partial(SchemaInfo, **kwargs), schema_vos)), total_count=total_count)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel
from spaceone.repository.model.repository_model import Repository

__all__ = ['Policy']


class Policy(MongoModel):
    policy_id = StringField(max_length=255, required=True, unique_with='domain_id')
    name = StringField(max_length=255)
    state = StringField(max_length=40, default='ENABLED', choices=('ENABLED', 'DISABLED'))
    permissions = ListField(StringField())
    labels = ListField(StringField(max_length=255))
    tags = DictField()
    repository = ReferenceField('Repository', reverse_delete_rule=DENY)
    repository_id = StringField(max_length=40)
    project_id = StringField(max_length=255, default=None, null=True)
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'name',
            'permissions',
            'labels',
            'tags'
        ],
        'minimal_fields': [
            'policy_id',
            'name'
        ],
        'change_query_keys': {
            'repository_id': 'repository.repository_id'
        },
        'reference_query_keys': {
            'repository': Repository
        },
        'ordering': ['name'],
        'indexes': [
            'policy_id',
            'repository',
            'repository_id',
            'project_id',
            'domain_id',
        ]
    }



import logging
from spaceone.repository.manager.plugin_manager import PluginManager
from spaceone.repository.connector.remote_repository_connector import RemoteRepositoryConnector

__all__ = ['RemotePluginManager']

_LOGGER = logging.getLogger(__name__)


class RemotePluginManager(PluginManager):
    """
    self.repository (=repository_vo)
    Remote Plugin make gRPC call to remote repository (like marketplace)
    If connector gets plugin_info, this is gRPC message.
    """

    def get_plugin(self, plugin_id, domain_id, only=None):
        conn = self._get_conn_from_repository()
        connector = self.locator.get_connector('RemoteRepositoryConnector', conn=conn)

        plugin_info = connector.get_plugin(plugin_id, only)
        return self._get_updated_plugin_info(plugin_info)

    def list_plugins(self, query):
        conn = self._get_conn_from_repository()
        connector = self.locator.get_connector('RemoteRepositoryConnector', conn=conn)

        response = connector.list_plugins(query, self.repository.repository_id)
        _LOGGER.debug(f'[remote list_plugin] count: {response.total_count}')

        for plugin_info in response.results:
            # Warning:
            # This is side effect coding, since plugin_vo is protobuf message
            self._get_updated_plugin_info(plugin_info)

        return response.results, response.total_count

    def stat_plugins(self, query):
        raise NotImplementedError('Remote repository is not supported.')

    def get_plugin_versions(self, plugin_id, domain_id):
        """ Get version of image

        version: tag list of docker image
        create RegistryConnector
        call get_tags()

        Returns:
            A list of docker tag
        """

        conn = self._get_conn_from_repository()
        connector = self.locator.get_connector('RemoteRepositoryConnector', conn=conn)

        versions = connector.get_plugin_version(plugin_id)
        _LOGGER.debug(f'[get_plugin_version] versions: {versions}')
        return versions

    def _get_conn_from_repository(self):
        conn = {'endpoint': self.repository.endpoint}
        return conn

    def _get_updated_plugin_info(self, plugin_info):
        """
        plugin_info is Protobuf Message
        We want to change our plugin_info (especially repository_info)

        Args:
            - plugin_info: protobuf message
        """
        # domain_id is remote repository's domain_id
        # change to local repository's domain_id  
        # There is no way to find domain_id
        # TODO: plugin_info.domain_id = self.repository.domain_id

        plugin_info.repository_info.name = self.repository.name
        plugin_info.repository_info.repository_type = self.repository.repository_type
        return plugin_info



import logging

from spaceone.core import config
from spaceone.core.error import *
from spaceone.repository.model import *
from spaceone.repository.manager.schema_manager import SchemaManager

__all__ = ['LocalSchemaManager']

_LOGGER = logging.getLogger(__name__)


class LocalSchemaManager(SchemaManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.schema_model: Schema = self.locator.get_model("Schema")

    def register_schema(self, params):
        def _rollback(schema_vo):
            schema_vo.delete()

        schema_vo = self.schema_model.create(params)
        self.transaction.add_rollback(_rollback, schema_vo)

        return schema_vo

    def update_schema(self, params):
        schema_vo = self.get_schema(params['name'], params['domain_id'])
        return self.update_schema_by_vo(params, schema_vo)

    def update_schema_by_vo(self, params, schema_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Schema Data : {old_data["name"]}')
            schema_vo.update(old_data)

        self.transaction.add_rollback(_rollback, schema_vo.to_dict())
        return schema_vo.update(params)

    def delete_schema(self, schema_name, domain_id):
        schema_vo = self.schema_model.get(name=schema_name, domain_id=domain_id)
        schema_vo.delete()

    def get_schema(self, schema_name, domain_id, only=None):
        if domain_id:
            schema_vo = self.schema_model.get(name=schema_name, domain_id=domain_id, only=only)
        else:
            schema_vo = self.schema_model.get(name=schema_name, only=only)

        return schema_vo

    def list_schemas(self, query):
        return self.schema_model.query(**query)

    def stat_schemas(self, query):
        return self.schema_model.stat(**query)



import logging

from spaceone.core.error import *
from spaceone.repository.model import *
from spaceone.repository.manager.policy_manager import PolicyManager

__all__ = ['LocalPolicyManager']

_LOGGER = logging.getLogger(__name__)


class LocalPolicyManager(PolicyManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.policy_model: Policy = self.locator.get_model("Policy")

    def create_policy(self, params):
        def _rollback(policy_vo):
            policy_vo.delete()

        policy_vo = self.policy_model.create(params)
        self.transaction.add_rollback(_rollback, policy_vo)

        return policy_vo

    def update_policy(self, params):
        policy_vo = self.get_policy(params['policy_id'], params['domain_id'])
        return self.update_policy_by_vo(params, policy_vo)

    def update_policy_by_vo(self, params, policy_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Policy Data : {old_data["name"]} ({old_data["policy_id"]})')
            policy_vo.update(old_data)

        self.transaction.add_rollback(_rollback, policy_vo.to_dict())
        return policy_vo.update(params)

    def delete_policy(self, policy_id, domain_id):
        policy_vo = self.policy_model.get(domain_id=domain_id, policy_id=policy_id)
        policy_vo.delete()

    def get_policy(self, policy_id, domain_id, only=None):
        if domain_id:
            policy_vo = self.policy_model.get(policy_id=policy_id, domain_id=domain_id, only=only)
        else:
            policy_vo = self.policy_model.get(policy_id=policy_id, only=only)

        return policy_vo

    def list_policies(self, query):
        return self.policy_model.query(**query)

    def stat_policies(self, query):
        return self.policy_model.stat(**query)



import logging

from spaceone.core.error import *
from spaceone.core.auth.jwt.jwt_util import JWTUtil

from spaceone.repository.connector.remote_repository_connector import RemoteRepositoryConnector
from spaceone.repository.manager.repository_manager import RepositoryManager

_LOGGER = logging.getLogger(__name__)


class RemoteRepositoryManager(RepositoryManager):

    def register_repository(self, params):
        """
        Args:
            params:
                - name
                - repository_type: remote
                - endpoint
                - version
                - secret_id
                - domain_id

        Connect to Remote Repository via secret_id
        Get repository_id of remote.
        use remote's repository_id as my repository_id
        """

        endpoint = params.get('endpoint')
        secret_id = params.get('secret_id')

        if endpoint is None:
            raise ERROR_REQUIRED_PARAMETER(key='endpoint')

        conn = {'endpoint': endpoint}

        # if secret_id:
        #     token = self.transaction.get_meta('token')
        #     if token is None:
        #         raise ERROR_AUTHENTICATE_FAILURE(message='Empty token provided.')
        #
        #     domain_id = self._get_domain_id_from_token(token)
        #     remote_token = self._get_secret(secret_id, domain_id)
        #     conn['token'] = remote_token['token']

        connector: RemoteRepositoryConnector = self.locator.get_connector('RemoteRepositoryConnector', conn=conn)
        repo_info = connector.get_remote_repository()
        # Overwrite repository_id to Remote one
        params['repository_id'] = repo_info.repository_id

        return self.repo_model.create(params)

    def _get_domain_id_from_token(self, token):
        decoded_token = JWTUtil.unverified_decode(token)
        return decoded_token['did']



import logging
import jsonschema

from spaceone.core.service import *
from spaceone.core import utils

from spaceone.repository.error import *
from spaceone.repository.manager.identity_manager import IdentityManager
from spaceone.repository.manager.schema_manager.local_schema_manager import LocalSchemaManager
from spaceone.repository.manager.schema_manager.remote_schema_manager import RemoteSchemaManager
from spaceone.repository.manager.repository_manager import RepositoryManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler(exclude=['get'])
@authorization_handler(exclude=['get'])
@mutation_handler
@event_handler
class SchemaService(BaseService):

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'service_type', 'schema', 'domain_id'])
    def create(self, params):
        """Create Schema

        Args:
            params (dict): {
                'name': 'str',
                'service_type': 'str',
                'schema': 'dict',
                'labels': 'list',
                'tags': 'dict',
                'project_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            schema_vo (object)
        """

        # Pre-condition Check
        self._check_schema(params['schema'])
        self._check_project(params.get('project_id'), params['domain_id'])
        self._check_service_type(params.get('service_type'))

        schema_mgr: LocalSchemaManager = self.locator.get_manager('LocalSchemaManager')

        # Only LOCAL repository can be registered
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        params['repository'] = repo_mgr.get_local_repository()
        params['repository_id'] = params['repository'].repository_id

        return schema_mgr.register_schema(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def update(self, params):
        """Update Schema. (local repo only)

        Args:
            params (dict): {
                'name': 'str',
                'schema': 'dict',
                'labels': 'list',
                'tags': 'dict'
                'domain_id': 'str'
            }

        Returns:
            schema_vo (object)
        """

        # Pre-condition Check
        self._check_schema(params.get('schema'))
        self._check_service_type(params.get('service_type'))

        schema_mgr: LocalSchemaManager = self.locator.get_manager('LocalSchemaManager')
        return schema_mgr.update_schema(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def delete(self, params):
        """Delete Schema (local repo only)

        Args:
            params (dict): {
                'name': 'str',
                'domain_id': 'str'
            }

        Returns:
            schema_vo (object)
        """
        schema_name = params['name']
        domain_id = params['domain_id']

        schema_mgr: LocalSchemaManager = self.locator.get_manager('LocalSchemaManager')
        return schema_mgr.delete_schema(schema_name, domain_id)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name'])
    @change_only_key({'repository_info': 'repository'})
    def get(self, params):
        """ Get Schema (local & remote)

        Args:
            params (dict): {
                'name': 'str',
                'repository_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            schema_vo (object)
        """
        schema_name = params['name']
        domain_id = params.get('domain_id')
        repo_id = params.get('repository_id')
        only = params.get('only')

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repo_vos = repo_mgr.get_all_repositories(repo_id)

        for repo_vo in repo_vos:
            _LOGGER.debug(f'[get] find at name: {repo_vo.name} '
                          f'(repo_type: {repo_vo.repository_type})')
            schema_mgr = self._get_schema_manager_by_repo(repo_vo)
            try:
                schema_vo = schema_mgr.get_schema(schema_name, domain_id, only)
            except Exception as e:
                schema_vo = None

            if schema_vo:
                return schema_vo

        raise ERROR_NO_SCHEMA(name=schema_name)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    @change_only_key({'repository_info': 'repository'}, key_path='query.only')
    @append_query_filter(['repository_id', 'name', 'service_type', 'project_id', 'domain_id'])
    @append_keyword_filter(['name', 'labels'])
    def list(self, params):
        """ List schemas (local or repo)

        Args:
            params (dict): {
                'repository_id': 'str',
                'name': 'str',
                'service_type': 'str',
                'project_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            schemas_vo (object)
            total_count
        """

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repository_id = params['repository_id']
        repo_vo = repo_mgr.get_repository(repository_id)

        schema_mgr = self._get_schema_manager_by_repo(repo_vo)
        query = params.get('query', {})

        return schema_mgr.list_schemas(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'repository_id'])
    @append_query_filter(['repository_id', 'domain_id'])
    @append_keyword_filter(['name', 'labels'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        repository_id = params['repository_id']
        repo_vo = repo_mgr.get_repository(repository_id)

        schema_mgr = self._get_schema_manager_by_repo(repo_vo)
        query = params.get('query', {})
        return schema_mgr.stat_schemas(query)

    def _get_schema_manager_by_repo(self, repo_vo):
        if repo_vo.repository_type == 'local':
            local_schema_mgr: LocalSchemaManager = self.locator.get_manager('LocalSchemaManager', repository=repo_vo)
            return local_schema_mgr
        else:
            remote_schema_mgr: RemoteSchemaManager = self.locator.get_manager('RemoteSchemaManager', repository=repo_vo)
            return remote_schema_mgr

    @staticmethod
    def _check_schema(schema):
        """
        Check json schema
        """
        if schema:
            try:
                jsonschema.Draft7Validator.check_schema(schema)
            except Exception as e:
                raise ERROR_INVALID_SCHEMA(key='schema')

    @staticmethod
    def _check_service_type(name):
        """
        service_type has format rule
        format:
            <service>.<purpose>
        example:
            identity.domain
            inventory.collector

        Raises:
            ERROR_INVALID_PARAMETER
        """
        if name:
            pass
            # idx = name.split('.')
            # if len(idx) != 2:
            #     raise ERROR_INVALID_PARAMETER(key='service_type', reason=f'{name} format is invalid.')

    def _check_project(self, project_id, domain_id):
        if project_id:
            identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
            identity_mgr.get_project(project_id, domain_id)



import functools
from google.protobuf.struct_pb2 import Struct
from spaceone.api.repository.v1 import policy_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.repository.model.policy_model import Policy
from spaceone.repository.info.repository_info import RepositoryInfo

__all__ = ['PolicyInfo', 'PoliciesInfo']


def PolicyInfo(policy_vo, minimal=False):
    info = {
        'policy_id': policy_vo.policy_id,
        'name': policy_vo.name
    }

    if not minimal:
        info.update({
            'state': policy_vo.state,
            'permissions': change_list_value_type(policy_vo.permissions),
            'labels': change_list_value_type(policy_vo.labels),
            'tags': change_struct_type(policy_vo.tags),
            'project_id': policy_vo.project_id,
            'domain_id': policy_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(policy_vo.created_at) or policy_vo.created_at,
            'updated_at': utils.datetime_to_iso8601(policy_vo.updated_at) or policy_vo.updated_at
            })
        # WARNING
        # Based on local_policy or remote_policy
        # vo has different repository or repository_info field
        if getattr(policy_vo, 'repository', None):
            info.update({
                'repository_info': RepositoryInfo(policy_vo.repository, minimal=True)})
        if getattr(policy_vo, 'repository_info', None):
            info.update({
                'repository_info': RepositoryInfo(policy_vo.repository_info, minimal=True)})

    return policy_pb2.PolicyInfo(**info)


def PoliciesInfo(policy_vos, total_count, **kwargs):
    return policy_pb2.PoliciesInfo(results=list(
        map(functools.partial(PolicyInfo, **kwargs), policy_vos)), total_count=total_count)



import logging

from spaceone.core.error import *
from spaceone.core.service import *
from spaceone.repository.manager import *

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class RepositoryService(BaseService):

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'repository_type'])
    def register(self, params):
        """
        Args:
            params:
                - name
                - repository_type: local | remote
                - endpoint
                - version
                - secret_id

        if repository_type == remote, do register remote repository
        """
        repo_type = params.get('repository_type')
        if repo_type == 'local':
            repo_mgr = self.locator.get_manager('LocalRepositoryManager')
        else:
            repo_mgr = self.locator.get_manager('RemoteRepositoryManager')

        return repo_mgr.register_repository(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    def update(self, params):
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        return repo_mgr.update_repository(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    def deregister(self, params):
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')

        return repo_mgr.delete_repository(params['repository_id'])

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['repository_id'])
    def get(self, params):
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        return repo_mgr.get_repository(params['repository_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @append_query_filter(['repository_id', 'name', 'repository_type'])
    @append_keyword_filter(['repository_id', 'name'])
    def list(self, params):
        repo_mgr: RepositoryManager = self.locator.get_manager('RepositoryManager')
        query = params.get('query', {})
        return repo_mgr.list_repositories(query)



import logging
from abc import abstractmethod

from spaceone.core.manager import BaseManager

__all__ = ['SchemaManager']

_LOGGER = logging.getLogger(__name__)


class SchemaManager(BaseManager):

    def create_schema(self, params):
        pass

    def update_schema(self, params):
        pass

    def delete_schema(self, schema_id, domain_id):
        pass

    @abstractmethod
    def get_schema(self, schema_id, domain_id, only=None):
        pass

    @abstractmethod
    def list_schemas(self, query):
        pass

    @abstractmethod
    def stat_schemas(self, query):
        pass



import logging
from abc import abstractmethod

from spaceone.core.manager import BaseManager

__all__ = ['PolicyManager']
_LOGGER = logging.getLogger(__name__)


class PolicyManager(BaseManager):

    def create_policy(self, params):
        pass

    def update_policy(self, params):
        pass

    def delete_policy(self, policy_id, domain_id):
        pass

    @abstractmethod
    def get_policy(self, policy_id, domain_id, only=None):
        pass

    @abstractmethod
    def list_policies(self, query):
        pass

    @abstractmethod
    def stat_policies(self, query):
        pass



from spaceone.repository.service.repository_service import *
from spaceone.repository.service.plugin_service import *
from spaceone.repository.service.schema_service import *
from spaceone.repository.service.policy_service import *
